import os

import click

from yhy import require
from yhy.core.img import clipboard_img, save


@click.option('-C', '--directory', type=click.Path(exists=False))
@click.option('--half', is_flag=True)
def command(directory, half):
    img, hexhash = clipboard_img(half=half)
    filename = f'{hexhash}.png'

    if not directory:
        path = filename
    else:
        require.directory(directory)
        path = os.path.join(directory, filename)

    save(img, path)
    click.echo(path)



from hashlib import sha1

import click
from PIL import ImageGrab
from PIL.Image import BICUBIC


def clipboard_img(half=False):
    img = ImageGrab.grabclipboard()

    if img is None:
        raise click.UsageError('Clipboard does not contain image data')

    # NOTE: Calcuate hash before modification like scaling down.
    # This is because the pasting image should be unique as a capture process,
    # not as a resulting image.
    ho = sha1()
    ho.update(img.tobytes())
    hexhash = ho.hexdigest()

    if half:
        img = scaled(img, 0.5)

    return (img, hexhash)


def save(img, path):
    img.save(path)


def scaled(img, s):
    w, h = img.size
    return img.resize((int(w*s), int(h*s)), resample=BICUBIC)



from hashlib import sha1
import os
import random

import click
from google.cloud import texttospeech
from prompt_toolkit import PromptSession

import pyperclip


@click.option('--anki-media', envvar='ANKI_MEDIA')
def command(anki_media):
    session = PromptSession()
    speak = make_speak()
    while True:
        text = session.prompt('TTS > ')
        text = text.strip()
        sound, ext = speak(text)
        path = path_for(anki_media, sound, ext)
        write(path, sound)
        sl = soundlink(path)
        print(sl)
        copy(f'{text}\n{sl}')
        play(path)


def make_speak():
    client = texttospeech.TextToSpeechClient()
    conf = texttospeech.types.AudioConfig(
        audio_encoding=texttospeech.enums.AudioEncoding.MP3)

    def speak(text):
        si = texttospeech.types.SynthesisInput(text=text)
        vo = pick_voice()
        response = client.synthesize_speech(si, vo, conf)
        return (response.audio_content, '.mp3')

    return speak


def pick_voice():
    V = texttospeech.types.VoiceSelectionParams
    voices = [
        V(language_code='en-US', name='en-US-Wavenet-A'),
        V(language_code='en-US', name='en-US-Wavenet-B'),
        V(language_code='en-US', name='en-US-Wavenet-C'),
        V(language_code='en-US', name='en-US-Wavenet-D'),
        V(language_code='en-US', name='en-US-Wavenet-E'),
        V(language_code='en-US', name='en-US-Wavenet-F'),
    ]
    return random.choice(voices)


def path_for(anki_media, sound, ext):
    name = name_for(sound)
    return os.path.join(anki_media, name + ext)


def name_for(sound):
    ho = sha1()
    ho.update(sound)
    return ho.hexdigest()


def write(path, sound):
    with open(path, 'wb') as f:
        f.write(sound)


def soundlink(path):
    name = os.path.basename(path)
    return f'[sound:{name}]'


def copy(s):
    pyperclip.copy(s)


def play(path):
    cmd = f"( afplay '{path}' & )"
    os.system(cmd)



import io

import click
from google.cloud import vision
from PIL import ImageGrab


@click.option('-l', '--lang', default='en')
def command(lang):
    client = vision.ImageAnnotatorClient()

    img = ImageGrab.grabclipboard()
    if img is None:
        raise click.UsageError('Clipboard does not contain image data')
    b = io.BytesIO()
    img.save(b, 'png')

    image = vision.types.Image(content=b.getvalue())
    image_context = vision.types.ImageContext(language_hints=[lang])
    response = client.text_detection(image=image, image_context=image_context)
    text = response.full_text_annotation.text
    text = text.strip()
    click.echo(text)
